//---------------------------------------------------------------------------
// JrDebug demo that demonstrates unit testing
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// Include the Debugging Library files
//  Note that only our main .cpp needs to include the jrdebug_main.cpp file
//  alternatively, you could add this cpp to your project or makefile.
#include "../../jrdebug_main.cpp"
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
/*
 * simple wildcard string matching
 *
 * Copyright (c) 2005 by Joergen Ibsen / Jibz
 * All Rights Reserved
 *
 * http://www.ibsensoftware.com/
 */
 
#include <stdio.h>
 
/**
 * Simple non-recursive wildcard string matcher.
 * @param pat - pattern to match against.
 * @param str - string to match.
 * @return 1 on match, 0 otherwise.
 * @note The pattern may contain the wildcards '?' and '*'.
 * '?' matches a single character. '*' matches zero or more
 * characters. The wildcard characters can be escaped using
 * '**' and '*?'.
 * @note Runs in worst case O(n*m) time, where n is the length
 * of str and m is the length of pat. Uses no extra space.
 */
int wildcard_match(const char *pat, const char *str)
{
    const char *p = pat;
    const char *s = str;
    int flag = 0;
 
    while (1)
    {
        if (*p == '\0')
        {
            /* if pattern and string are both empty, it's a match */
            if (*s == '\0') return 1;
 
            /* if before first *, segment could only match here */
            if (!flag) return 0;
 
            /* reset p to start of current segment, and advance str */
            p = pat;
            s = ++str;
        }
 
        if (*p == '?')
        {
            /* a ? must match a character */
            if (*s == '\0') return 0;
 
            /* advance p and s */
            ++p;
            ++s;
 
            /* continue loop from start */
            continue;
        }
 
        if (*p == '*')
        {
            ++p;
 
            /* check this is not an escaped wildcard */
            if (*p != '*' && *p != '?')
            { 
                /* point pat to the next segment */
                pat = p;
 
                /* if empty, the * matches the rest of s */
                if (*p == '\0') return 1;
 
                /* point str to the unmatched part of string */
                str = s;
 
                /* indicate we have passed a * wildcard */
                flag = 1;
            }
        }
 
        /* there must be a character to match here */
        if (*s == '\0') return 0;
 
        if (*p++ != *s++)
        {
            /* if before first *, segment could only match here */
            if (!flag) return 0;
 
            /* reset p to start of current segment, and advance str */
            p = pat;
            s = ++str;
        }
    }
}
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
void UnitTest_JibzWildCard()
{
    dbtest(wildcard_match("*.*", "file.dat")==1);
    dbtest(wildcard_match("*.*", ".")==1);
    dbtest(wildcard_match("*.*", "x.")==1);
    dbtest(wildcard_match("*.*", ".x")==1);
    dbtest(wildcard_match("*.*", "x")==0);
 
    dbtest(wildcard_match("foo*", "foo")==1);
    dbtest(wildcard_match("foo*", "fo")==0);
    dbtest(wildcard_match("foo*", "foobar")==1);
    dbtest(wildcard_match("foo*", "afoo")==0);
    dbtest(wildcard_match("foo*", "afo")==0);
    dbtest(wildcard_match("foo*", "afoobar")==0);
 
	dbtestinfo("mouser bug",wildcard_match("foo*", "afoobar")==1);

    dbtest(wildcard_match("*f?o*d", "fod")==0);
    dbtest(wildcard_match("*f?o*d", "food")==1);
    dbtest(wildcard_match("*f?o*d", "foxd")==0);
    dbtest(wildcard_match("*f?o*d", "fxod")==1);
    dbtest(wildcard_match("*f?o*d", "fxodfnod")==1);
    dbtest(wildcard_match("*f?o*d", "fnoxdf")==0);
    dbtest(wildcard_match("*f?o*d", "abcfxonopd")==1);
 
    dbtest(wildcard_match("***", "*")==1);
    dbtest(wildcard_match("***", "abc*")==0);
    dbtest(wildcard_match("***", "*abc")==1);
    dbtest(wildcard_match("*?*", "*?*")==0);
    dbtest(wildcard_match("*?*", "?abc")==1);
    dbtest(wildcard_match("*?*", "abc?")==0);
}

// register the wildcard test
JrDebugRegisterTestFunction("wildcards","Test Wildcard Matches",UnitTest_JibzWildCard,"");
//---------------------------------------------------------------------------






//---------------------------------------------------------------------------
int main(int argc, char *argv[])
{
	// automatically parse commandline options and exit if appropriate
	if (JrDebugParseCommandlineArgc("Demo10 (wildcard function test app)",&argc,argv,"-dbt"))
		return 0;

	// announce
	printf("Unit Test Demo program.  Get help with -dbh commandline option.\n");

	// all done
	return 0;
}
//---------------------------------------------------------------------------
